<?php

session_start();

require_once __DIR__ . '/rbs.php';
define('RBS_MODULE_VERSION' , '2.1');

class Rbspayment_PaymentSystemDriver extends AMI_PaymentSystemDriver
{

    /**
     * Get checkout button HTML form
     *
     * @param array $aRes Will contain "error" (error description, 'Success by default') and "errno" (error code, 0 by default). "forms" will contain a created form
     * @param array $aData The data list for button generation
     * @param bool $bAutoRedirect If form autosubmit required (directly from checkout page)
     * @return bool true if form is generated, false otherwise
     */
    public function getPayButton(array &$aRes, array $aData, $bAutoRedirect = false)
    {

        $aRes['errno'] = 0;
        $aRes['error'] = 'Success';

        // Format fields
        foreach (Array("return", "description") as $fldName) {
            $aData[$fldName] = htmlspecialchars($aData[$fldName]);
        }
        $aData['hiddens'] = $this->getScopeAsFormHiddenFields($aData);

        // Disable to process order using example button
        // $aData["disabled"] = "disabled";

        // Set your fields of $aData here

        return parent::getPayButton($aRes, $aData, $bAutoRedirect);
    }

    /**
     * Get the form that will be autosubmitted to payment system. This step is required for some shooping cart actions.
     *
     * @param array $aData The data list for button generation
     * @param array $aRes Will contain "error" (error description, 'Success by default') and "errno" (error code, 0 by default). "forms" will contain a created form
     * @return bool true if form is generated, false otherwise
     */
    public function getPayButtonParams(array $aData, array &$aRes)
    {

        $oEshopCart = AMI::getResource('eshop/cart');

        // Get cart total amount
        $aTotal = $oEshopCart->getTotal();

        $amount = $aData['amount'] * 100;

        // Get array of cart items objects
        $aCartItems = $oEshopCart->getItems();

        $config['user_name'] = $aData['username'];
        $config['password'] = $aData['password'];
        $config['two_stage'] = ($aData['two_stage'] == 'true') ? TRUE : FALSE;
        $config['test_mode'] = ($aData['test_mode'] == 'true') ? TRUE : FALSE;
        $config['send_order'] = ($aData['send_check'] == 'true') ? TRUE : FALSE;
        $config['tax_system'] = $aData['tax_system'];

        $config['ffd_version'] = $aData['ffd_version'];
        $config['ffd_paymentMethodType'] = $aData['ffd_paymentMethodType'];
        $config['ffd_paymentObjectType'] = $aData['ffd_paymentObjectType'];

        // Check parameters and set your fields here
        $aRes['errno'] = 0;
        $aRes['error'] = 'Success';

        $config['cms_name'] = 'AmiroCMS';
        $config['cms_version'] = '';
        $config['module_version'] = RBS_MODULE_VERSION;


        $rbs = new RBS($config);

        $rbs->logging = true;

        // IF send check collect orderBundle
        if ($rbs->send_order) {

            foreach ($aCartItems as $oCartItem) {

                $id = $oCartItem->getItemId();

                // так как разрабы Амиро(RU) любят выдавать чужие плагины за свои, этот коммент здесь уместен ;)
                $aColumn = AMI::getSingleton('db')->fetchCol("SELECT `name` FROM `cms_es_items` WHERE `id` = " . $id);
                foreach ($aColumn as $value) {
                    $i[$id]['name'] = $value;
                }
                // Get cart item price info
                $i[$id]['price'] = $oCartItem->getPriceInfo();
                // Get cart item quantity info
                $i[$id]['quantity'] = $oCartItem->getQty();
                // Get cart item tax values
                $i[$id]['tax'] = $oCartItem->getTax();
                // Get cart item discount values
                $i[$id]['discount'] = $oCartItem->getDiscount();
                // Get cart item shipping values
                $i[$id]['shipping'] = $oCartItem->getShipping();

                // $items[$id]['item'] = $oCartItem->getItem();
            }

            $items = array();
            $itemsCnt = 1;
            $new_order_cost = 0;
            foreach ($i as $k => $value) {

                // Get item TAX
                $item_rate = $value['tax']['tax'];

                if ($item_rate == 20) {
                    $tax_type = 6;
                } else if ($item_rate == 18) {
                    $tax_type = 3;
                } else if ($item_rate == 10) {
                    $tax_type = 2;
                } else if ($item_rate == 0) {
                    $tax_type = 1;
                } else {
                    $tax_type = 0;
                }

                // main data
                $item['positionId'] = $itemsCnt++;
                $item['name'] = $value['name'];
                $item['quantity'] = array(
                    'value' => $value['quantity'],
                    'measure' => 'piece'
                );

                $price = $value['price']['price'] * 100;

                $item['itemAmount'] = $price * $value['quantity'];
                $item['itemPrice'] = $price;
                $item['itemCode'] = $k;
                $item['tax'] = array(
                    'taxType' => $tax_type
                );

                $new_order_cost += $item['itemAmount'];

                // FFD 1.05 added
                if ($config['ffd_version'] == 'v105') {

                    $attributes = array();
                    $attributes[] = array(
                        "name" => "paymentMethod",
                        "value" => $config['ffd_paymentMethodType']
                    );
                    $attributes[] = array(
                        "name" => "paymentObject",
                        "value" => $config['ffd_paymentObjectType']
                    );

                    $item['itemAttributes']['attributes'] = $attributes;
                }

                $items[] = $item;
            }

            // DELIVERY POSITION - привет Амиро =)
            $delivery = $amount - $new_order_cost;

            if ($delivery > 0) {
                $itemShipment['positionId'] = $itemsCnt;
                $itemShipment['name'] = 'Доставка';
                $itemShipment['quantity'] = array(
                    'value' => 1,
                    'measure' => 'piece'
                );
                $itemShipment['itemAmount'] = $itemShipment['itemPrice'] = $delivery;
                $itemShipment['itemCode'] = 'Delivery';
                $itemShipment['tax'] = array(
                    'taxType' => 0 //todo get if from delivery
                );

                // FFD 1.05 added
                if ($config['ffd_version'] == 'v105') {

                    $attributes = array();
                    $attributes[] = array(
                        "name" => "paymentMethod",
                        "value" => $config['ffd_paymentMethodType']
                    );
                    $attributes[] = array(
                        "name" => "paymentObject",
                        "value" => 4
                    );

                    $itemShipment['itemAttributes']['attributes'] = $attributes;
                }
                $items[] = $itemShipment;
            }

            /* Создание и заполнение массива данных заказа для фискализации */
            $order_bundle = array(
                'orderCreationDate' => time(),
                'customerDetails' => array(
                    'email' => $aData['email'],
                ),
                'cartItems' => array('items' => $items)
            );
            $rbs->order_bundle = $order_bundle;
      }

        $order_number = $aData['order_id'];

        $return_url = $GLOBALS['ROOT_PATH_WWW'] . '_local/eshop/rbspayment.php';

        $aData['error'] = FALSE;
        $response = $rbs->register_order($order_number, $amount, $return_url);

        if (is_array($response) && !isset($response['errorCode'])) {
            $oSession = AMI::getSingleton('env/session');
            if (!$oSession->isStarted()) {
                $oSession->start();
            }

            $oSession->{$response['orderId'] . '_number'} = $order_number;
            $oSession->{$response['orderId'] . '_return'} = $aData['return'];
            $oSession->{$response['orderId'] . '_cancel'} = $aData['cancel'];
            $oSession->{'user_name'} = $config['user_name'];
            $oSession->{'password'} = $config['password'];
            $oSession->{'logging'} = $rbs->logging;
            $oSession->{'test_mode'} = $config['test_mode'];
            $aData['url'] = $response['formUrl'];
            $aData['order_id'] = $response['orderId'];
            $aData['request_method'] = 'GET';
        } else {
            $aData['error'] = TRUE;
            $aData['url'] = $aData['cancel'];
            if (is_array($response) && isset($response['errorMessage'])) {
                $aData['message'] = $response['errorMessage'];
            }
        }
        return parent::getPayButtonParams($aData, $aRes);
    }


    /**
     * Verify the order from user back link. In success case 'accepted' status will be setup for order.
     *
     * @param array $aGet $_GET data
     * @param array $aPost $_POST data
     * @param array $aRes reserved array reference
     * @param array $aCheckData Data that provided in driver configuration
     * @param array $aOrderData order data that contains such fields as id, total, order_date, status
     * @return bool true if order is correct and false otherwise
     * @see AMI_PaymentSystemDriver::payProcess(...)
     */
    public function payProcess(array $aGet, array $aPost, array &$aRes, array $aCheckData, array $aOrderData)
    {
        // See implplementation of this method in parent class

        return parent::payProcess($aGet, $aPost, $aRes, $aCheckData, $aOrderData);
    }

    /**
     * Verify the order by payment system background responce. In success case 'confirmed' status will be setup for order.
     *
     * @param array $aGet $_GET data
     * @param array $aPost $_POST data
     * @param array $aRes reserved array reference
     * @param array $aCheckData Data that provided in driver configuration
     * @param array $aOrderData order data that contains such fields as id, total, order_date, status
     * @return int -1 - ignore post, 0 - reject(cancel) order, 1 - confirm order
     * @see AMI_PaymentSystemDriver::payCallback(...)
     */
    public function payCallback(array $aGet, array $aPost, array &$aRes, array $aCheckData, array $aOrderData)
    {
        // See implplementation of this method in parent class
        die('3');
        return parent::payCallback($aGet, $aPost, $aRes, $aCheckData, $aOrderData);
    }

    /**
     * Return real system order id from data that provided by payment system.
     *
     * @param array $aGet $_GET data
     * @param array $aPost $_POST data
     * @param array $aRes reserved array reference
     * @param array $aAdditionalParams reserved array
     * @return int order Id
     * @see AMI_PaymentSystemDriver::getProcessOrder(...)
     */
    public function getProcessOrder(array $aGet, array $aPost, array &$aRes, array $aAdditionalParams)
    {
        // See implplementation of this method in parent class

        return parent::getProcessOrder($aGet, $aPost, $aRes, $aAdditionalParams);
    }
}
